#!/usr/bin/env python
#
#  Copyright (c) 2013 Dell, Inc.
#  by Praveen K Paladugu <praveen_paladugu@dell.com>
#  Licensed under the GNU General Public license, version 2 or higher.
#


""" Class for the EULA Text Box"""

import sys, os
import logging
import App_Conf_Parser
import DialogBox
import gettext

gettext.textdomain('dell_app_init')
_ = lambda x: gettext.ldgettext("dell_app_init", x)

## X server should be up by now. So, the following import should be fine
try:
    import gtk
    import gtk.glade
except:
    print "Gtk/Glade is not installed."
    sys.exit(1)


eulabox_glade_path="/usr/share/dell_app_init/EulaBox.glade"

class EulaBox:

    conf_parser = 0
    eula_tree = 0
    eula_window = 0
    logger = 0
    eula_accepted = 0
    radio_disagree = None


    def __init__(self, filename, parser):
        """ Initialize the EulaBox with the given filename """

	self.logger = logging.getLogger("startup_app.EulaBox")

	# Capture the parser object.
	self.conf_parser = parser

	self.eula_accepted = 0
	self.eula_tree = gtk.glade.XML(eulabox_glade_path)
	self.eula_tree.signal_autoconnect(self)

	self.logger.debug (_("EulaBox's glade path = %s"), eulabox_glade_path)


        self.radio_disagree = self.eula_tree.get_widget("LicenseDeny_radio")

	self.eula_window = self.eula_tree.get_widget("License Agreement")
	self.eula_window.connect("destroy",self.on_destroy_event)
        self.eula_window.connect("delete_event",self.on_delete_event)


	eula_textbox = self.eula_tree.get_widget("EulaTextBox")

	self.logger.debug(_("Eula's text filename = %s "),filename)
	fp = open (filename)
	#Negative parameter reads to the end of the file
	eula_text = fp.read(-1)
	fp.close()

	# Populate the buffer and set to text box's buffer
	text_buffer = gtk.TextBuffer()
	text_buffer.set_text(eula_text)
	eula_textbox.set_buffer(text_buffer)

        return

    def  grab_focus (self):
        """ Let the Accept radio button grab the focus """
        self.radio_disagree.grab_focus()
        return



    def is_radio_license_accepted(self):
	agree_radio = self.eula_tree.get_widget("LicenseAgree_radio")
	return agree_radio.get_active()


    def on_Forward_clicked (self, widget):

	if (not self.is_radio_license_accepted()):

	    # If License Not accepted, show the eula Dialog.
	    #self.eula_window.hide_all()
	    msg = _("End User License Agreement must be accepted to use the appliance.\nIf you reject the EULA, the appliance will be shut down.\nDo you want to continue?")
	    db = DialogBox.DialogBox(self.eula_window,msg,buttons=gtk.BUTTONS_OK_CANCEL)
	    rc = db.display()
	    if rc == gtk.RESPONSE_OK:
		self.logger.info (_("Shutting down the appliance, as Eula was not accepted."))
		self.eula_accepted = 0
		self.close_eula_and_return()
	    else:
		self.logger.debug (_("Redisplay the Eula Window"))
		self.eula_window.show_all()

	else:
	    # If the EULA is accepted
	    self.save_eula_state()
	    self.eula_accepted = 1
	    self.close_eula_and_return()

    	return

    def save_eula_state(self):

	eula_state = self.conf_parser.get_conf_val ("app_conf", "eula_state")
	if (eula_state == "" ):
            ##Use the default setting
	    eula_state = self.conf_parser.get_conf_val("DEFAULT", "eula_state")

	eula_input = self.conf_parser.get_conf_val ("app_conf","eula_input_version_file")
	
	installed_version = self.conf_parser.get_eula_version_from_file (eula_input) 

	if (installed_version == -1 ):
	    # Either the eula_input file is not present or empty, capture the installed version as 0
	    installed_version = "0.0"

	dir_name = os.path.dirname(eula_state)	
	if not  os.path.exists(dir_name):
	    os.makedirs(dir_name, 0744)	

	fd = open(eula_state, 'w')
	if fd:
    	    fd.write (str(installed_version))
	    fd.close
	else:
	    self.logger.error(_("Not able to open the file to save EULA state"))	    

	return


    def is_eula_accepted(self):
	## Return the saved state of eula_accepted variable.
	if (self.eula_accepted):
	    return 1
	else:
	    return 0
	

    def close_eula_and_return(self):
    
        self.eula_window.hide_all()
        gtk.main_quit()

	return
	
    def on_destroy_event (self, widget):
	# Ignore destroy event
	return

    def on_delete_event (self, widget,data):
	# Ignore the delete event
        return True

