#!/usr/bin/env python
#
#  Copyright (c) 2014 Dell, Inc.
#  by Praveen K Paladugu <praveen_paladugu@dell.com>
#  Licensed under the GNU General Public license, version 2 or higher.
#

import sys
import logging
import subprocess
import DialogBox
import gettext
import shlex
import os
import platform

gettext.textdomain('dell_app_init')
_ = lambda x: gettext.ldgettext("dell_app_init", x)

## X server should be up by now. So, the following import should be fine
try:
    import gtk
    import gtk.glade
except:
    print "Gtk/Glade is not installed."
    sys.exit(1)

gtk.glade.bindtextdomain('dell_app_init','/usr/share/locale')
gtk.glade.textdomain('dell_app_init')

class  Hostname_Win:

    hn_entry = None
    hn_window = None

    def __init__(self, parser=""):

	self.logger = logging.getLogger("startup_app.HostName")
        hostname_glade_path =  "/usr/share/dell_app_init/Hostname_Win.glade"
        self.logger.debug (_("Glade file of Hostname Window = %s"), hostname_glade_path )

        ## load the Glade file
        hostname_xml = gtk.glade.XML(hostname_glade_path,"Hostname_Win")
	self.hn_entry = hostname_xml.get_widget("Hostname_Entry")
	self.hn_entry.set_text (self.get_hostname())
	self.hn_window = hostname_xml.get_widget("Hostname_Win")

	hostname_xml.signal_autoconnect(self)

	self.parser = parser

	return


    def get_hostname(self):
	""" Get the hostname without any resolution checks"""

	cmd = 'hostname'
        cmd_out = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
        host_name = cmd_out.communicate()[0]
        # Remove the trailing whitespace chars
        host_name = host_name.strip()
        cmd_out.stdout.close()
	
	return host_name


    def on_Close_clicked(self,widget):
	""" Update the Hostname now"""
	cur_hn = self.get_hostname()
	new_hn = self.hn_entry.get_text()

	if cur_hn == new_hn:
	    msg = _("The hostname was not updated successfully.")
	    self.logger.info (msg)
	    db = DialogBox.DialogBox(self.hn_window,msg,buttons=gtk.BUTTONS_OK)
	    rc = db.display()
	    if rc == gtk.RESPONSE_OK:
		self.hn_window.hide_all()
		gtk.main_quit()
	else:
	    # Validate the input hostname before updating
#	    if new_hn.endswith("."):
#		new_hn = new_hn[:-1]
	    if not self.is_valid_hostname(new_hn):
		msg = _("The new hostname is not of acceptable format.\nPlease try a different hostname.")
		db = DialogBox.DialogBox(self.hn_window,msg,buttons=gtk.BUTTONS_OK)
		rc = db.display()
		## No matter what is returned from the dialog, return
		self.hn_window.show_all()
		return

	    self.set_hostname(new_hn)
	    msg = _("The hostname was updated successfully.")
	    db = DialogBox.DialogBox(self.hn_window,msg,buttons=gtk.BUTTONS_OK)
	    rc = db.display()
	    if rc == gtk.RESPONSE_OK:
		self.hn_window.hide_all()
		gtk.main_quit()

    def on_Hostname_destroy(self, widget):
	"""Handler for destroying the Hostname Window"""
	# Do not update the hostname and return.
	msg = _("The hostname was not updated successfully.")
        db = DialogBox.DialogBox(self.hn_window,msg,buttons=gtk.BUTTONS_OK)
        rc = db.display()
	self.hn_window.hide_all()
        gtk.main_quit()

    def is_valid_hostname (self, hostname):
        """ Check if the hostname is valid """
        import re

        ## Below pattern checking from /usr/share/system-config-network/netconfpkg/
        ##NC_functions.py in system-config-network pkg
        hostname_pattern = re.compile('^([a-zA-Z]|[0-9])(([a-zA-Z]|[0-9]|-)*([a-zA-Z]|[0-9]))?(\.([a-zA-Z]|[0-9])(([a-zA-Z]|[0-9]|-)*([a-zA-Z]|[0-9]))?)*$')
        length_pattern = re.compile('^[a-zA-Z0-9-_]{1,64}(\.([a-zA-Z0-9-_]{1,64}))*$')

        if not hostname:
            return False

        if (len(hostname)) < 256:
        # according to RFC 952 <name>  ::= <let>[*[<let-or-digit-or-hyphen>]<let-or-digit>]
        # & RFC 1123 - allows trailing and starting digits
        # ip like hostname are not allowed
            if not length_pattern.match(hostname):
                return False
            if not hostname_pattern.match(hostname):
                return False
        return True


    @staticmethod
    def get_resolvable_hostname():
        """Get the hostname and check if it can be resolved. If not, return the ip address"""
	import socket
	import subprocess


        host_name = socket.getfqdn()
        # Remove the trailing whitespace chars
        host_name = host_name.strip()
	current_ip = Hostname_Win.get_ip_address()

        if not 'localhost' in host_name:

            try:
                ip = socket.gethostbyname (host_name)
		if ip == current_ip:
		    return host_name
		else:
		    return current_ip
            except socket.gaierror, err:
                return current_ip

        else:
            return Hostname_Win.get_ip_address()


    @staticmethod
    def get_ip_address():
	""" Get the ip address of the first interface in the appliance """
    
        cmd = 'ip link show|grep -e"^[0-9]*:" | awk \'{print $2}\' | grep -v lo '
        cmd_out = subprocess.Popen(cmd,stdout=subprocess.PIPE, shell=True)
        interface_list = cmd_out.communicate()[0]
        cmd_out.stdout.close()
        interface_list = interface_list.replace(':','')
        first_interface = shlex.split(interface_list)[0]


        cmd = "ip add show dev " +first_interface
        cmd = cmd + " | grep inet | grep -v inet6 | awk '{print $2}' | cut -d / -f 1"

        cmd_out = subprocess.Popen(cmd,stdout=subprocess.PIPE, shell=True)
        ip_address = cmd_out.communicate()[0]
        ip_address = ip_address.strip()
        cmd_out.stdout.close()

        return ip_address
	

    def set_hostname(self,new_hostname):
	""" Function to update the hostname of the system."""
	
        if platform.linux_distribution()[1][0]=='7': #hostnamectl for centos 7
	    cmd = 'hostnamectl set-hostname ' + new_hostname
            cmd_out = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
            error_output = cmd_out.stdout.read()
	    cmd_out.communicate()
	    return_code = cmd_out.returncode
	    cmd_out.stdout.close()

            if return_code != 0:
                msg = _("Issue setting hostname: Debugging statement: " +error_output)
                db = DialogBox.DialogBox(self.hn_window,msg,buttons=gtk.BUTTONS_OK)
                rc = db.display()
                ## No matter what is returned from the dialog, return
                self.hn_window.show_all()
                return


        else: #centos 6  
	    cmd = "hostname "+ new_hostname
	    cmd_out = subprocess.Popen(cmd,stdout=subprocess.PIPE, shell=True)
	    cmd_out.stdout.close()
	
	    network_conf_file = "/etc/sysconfig/network"

	    if (not os.path.exists(network_conf_file) ):
	        # If the file doesn't exist
	        nw_fd = open( network_conf_file, 'w')
	        setting = "HOSTNAME=" + new_hostname
	        nw_fd.write(setting)
	        nw_fd.write("\n")
	        nw_fd.close ()

	    else:
	        # if the file exists, add/update the  entry 
	        nw_fd = open( network_conf_file, 'r')
	        if nw_fd:
		    nw_lines = nw_fd.readlines()
		    nw_fd.close()

	        ## Write back the lines just replacing the value of HOSTNAME variable.
	        nw_fd_wr = open( network_conf_file, 'w')
	        hn_setting_found = 0	

	        for line in nw_lines:
		    if line[:9] == 'HOSTNAME=':
		        hn_setting_found = 1
		        self.logger.debug (_('Replacing the value of HOSTNAME in network conf file'))
		        setting = "HOSTNAME=" + new_hostname
		        nw_fd_wr.write( setting)
		        nw_fd_wr.write("\n")
		    elif line == "\n":
		        pass
		    else:
		        nw_fd_wr.write(line)
		        nw_fd_wr.write("\n")
	    
	        if hn_setting_found == 0:
    		    setting = "HOSTNAME="+new_hostname
		    self.logger.debug (_('Addig the HOSTNAME value to network conf file'))
		    nw_fd_wr.write(setting)
		    nw_fd_wr.write("\n")

	        nw_fd_wr.close()

	    self.network_restart()
	self.run_post_script()
	return

    def network_restart(self):
	""" After the hostname is configured, restart the network service"""
	cmd = "service network restart >/dev/null 2>&1"
	cmd_out = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
	cmd_out.stdout.close()
	return
    
	
    def on_Hostname_destroy(self, widget):
	""" Handler for a destroy event"""
	self.hn_window.hide_all()
	gtk.main_quit()

    def run_post_script(self):
	
	post_script = self.parser.get_conf_val ("hostname","post_hostname_script")
	
        if post_script == "":
            return

        else:

	    cmd = shlex.split(post_script)
	    if self.parser.get_conf_val("hostname","hostname_to_script") == 'yes':
		cmd.append( self.hn_entry.get_text() )

	    cmd_out = subprocess.Popen(cmd , stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=False)
            self.logger.info ("Output from the Post Hostname Script = \n%s", cmd_out.communicate()[0])
            return

 

#if __name__ == '__main__':
#	hn = Hostname_Win()
#	print "Direct = ",hn.get_hostname()
#	print "Resolvable =", hn.get_resolvable_hostname#()
#	hn.hn_window.show_all()
#	gtk.main()

