#!/usr/bin/env python
#
#  Copyright (c) 2013 Dell, Inc.
#  by Praveen K Paladugu <praveen_paladugu@dell.com>
#  Licensed under the GNU General Public license, version 2 or higher.
#


import logging
import os, sys
import subprocess
import App_Conf_Parser
import DialogBox
import gettext
import Hostname_Win
import gobject

gettext.textdomain('dell_app_init')
_ = lambda x: gettext.ldgettext("dell_app_init", x)

## X server should be up by now. So, the following import should be fine
try:
    import gtk
    import gtk.glade
except:
    print "Gtk/Glade is not installed."
    sys.exit(1)

gtk.glade.bindtextdomain('dell_app_init','/usr/share/locale')
gtk.glade.textdomain('dell_app_init')



class Startup_Interface:

#Global vars in this class	
    topWindow = 0
    logger = 0
    EmptyWindow = 0
    EulaDisplayWindow =0
    EulaTextView = 0
    parser = 0
    reboot_appliance = False
    date_button = None
    __topwindow_tree_xml = None
    appliance_reset_initiated = 0


##TODO: Erorr out on GUI, if any of class imports fails.
##TODO: Where ever necessary, add debug logs

    def on_DateWindow_clicked(self, widget):
	""" Handler function to configure Date & Time """
	
	if not "/usr/share/system-config-date" in sys.path:
	    sys.path.append("/usr/share/system-config-date")
	self.topWindow.hide()
	import scdMainWindow
	reload(scdMainWindow)
	scdWindow = scdMainWindow.scdMainWindow(0)
	scdWindow.win.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
	scdWindow.closeParent = False

        import signal
	def interrupt_handler(signum, frame):
            signal.signal(signal.SIGINT, signal.SIG_DFL)

        signal.signal(signal.SIGINT, interrupt_handler)
        # When cancelling after pressing OK, causes a interrupt to occur which crashes the program
        # To prevent this, we can catch the interrupt and continue with the program
        try:
	    scdWindow.stand_alone()
        finally:
            # Restore original sigint so we can break out for developing purposes
            signal.signal(signal.SIGINT, signal.SIG_DFL)

	## After returning from the Date/Time Settings, run the related script.
	dt_script = self.parser.get_conf_val ("date_time", "date_time_script")
	if not dt_script == "":
	    cmd_out = subprocess.Popen(dt_script ,stdout=subprocess.PIPE,stderr=subprocess.STDOUT, shell=True)
	    cmd_out.wait() # Wait for the script to complete running.
	    
	scdWindow.win.hide()
	self.topWindow.show_all()
	return

    def on_HostnameButton_clicked(self, widget):
	""" Handler for updating the hostname of the appliance"""

	self.topWindow.hide_all()

	import Hostname_Win

	hn_win = Hostname_Win.Hostname_Win(parser = self.parser)

	hn_win.hn_window.show_all()
	gtk.main()

	hn_win.hn_window.hide_all()
	self.set_title_and_desc()
	self.topWindow.show_all()

    def on_Passwd_clicked (self,widget):
	""" Handler function to updating the admin password"""
    
	self.topWindow.hide_all()
	import Password_Win

        pw = Password_Win.Password_Win(parser = self.parser)

	pw.passWindow.show_all()
	gtk.main()

	pw.passWindow.hide_all()
	self.topWindow.show_all()

    def on_Reboot_clicked(self,widget):
	""" Exit the application and reboot the Appliance now"""
	msg = _("Are you sure you want to reboot the appliance?")
	db = DialogBox.DialogBox(self.topWindow,msg)
	rc = db.display()

	if rc == gtk.RESPONSE_YES:
	    self.reboot_appliance = True
	    gtk.main_quit()
	else:
	    self.topWindow.show_all()
	

    def on_EulaDisplayWindow_destroy_event (self, widget, data=None ):
	""" Destroy the EULA display and get back to Main Window """
        self.EulaDisplayWindow.hide_all()
        self.topWindow.show_all()
	
	return
    
    def on_EulaDisplayWindow_delete_event (self, widget, event, data=None):
	""" Closed the EULA Text Window with Alt-F4 """
        self.EulaDisplayWindow.hide_all()
        self.topWindow.show_all()
	
	# if set to False, the topWindow is also closed, which causes the program to mis-behave.
	return True
    
    def on_EULAText_clicked(self, widget):
	""" Handler for EULA Text Button. Show the EULA Text for customers to review """

	eula_text_filename = self.parser.get_eula_text_filename()
	self.logger.debug(_("filename of the EULA file = %s"), eula_text_filename)

	if ( eula_text_filename ) and os.access (eula_text_filename, os.R_OK):
	    # Open the file and read the text to a buffer. 
	    fp = open (eula_text_filename)
	    eula_text = fp.read(-1)
	    fp.close()
	
	    #Populate a TextBuffer and assign it the EulaDisplayWindow
	    eula_text_buffer = gtk.TextBuffer()
	    eula_text_buffer.set_text(eula_text)
	    self.EulaTextView.set_buffer(eula_text_buffer)
	    
	else:
	    #TODO: Throw an Error when the EULA file is not available.
	    #dell_app_init should fail if the file doesn't exist. But better to check here as well.
	    pass

	self.topWindow.hide_all()
	self.EulaDisplayWindow.show_all()
	return

    def on_EulaTextClose_clicked (self, widget):
	""" Close the EULA Text Display Window and redisplay the TopWindow"""
	
	self.EulaDisplayWindow.hide_all()
	self.topWindow.show_all()
	return

    def on_Language_clicked(self, widget):
	"""Handler function to configure Language"""

	if not "/usr/share/system-config-language" in sys.path:
	    sys.path.append("/usr/share/system-config-language")

	self.set_supported_languages()

	self.topWindow.hide()
	import language_gui
	app = language_gui.childWindow()
	#app.mainWindow.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_MENU)
	app.stand_alone()

	app.mainWindow.hide()
	self.topWindow.show_all()
	#TODO: Capture the language that is selected
	return
    
    def	set_supported_languages(self):
	""" Function to set the supported language setting in /etc/locale.conf file"""
	locale_conf_file = '/etc/sysconfig/i18n'	

	if (not os.path.exists(locale_conf_file) ):
	    locale_fd = open( locale_conf_file, 'w')
	    setting = "SUPPORTED=" + self.parser.get_conf_val ("app_conf", "supported_languages")
	    locale_fd.write(setting)
	    locale_fd.write("\n")
            #Set a default LANG setting as well.
            setting ='LANG="en_US.UTF-8"'
            locale_fd.write(setting)
	    locale_fd.write("\n")

	    locale_fd.close ()

	else:
	    ## If the locale exists, read all lines
	    locale_fd = open( locale_conf_file, 'r')
	    if locale_fd:
		loc_lines = locale_fd.readlines()
		locale_fd.close()

	    ## Write back the lines just replacing the value of SUPPORTED variable.
	    locale_fd_wr = open( locale_conf_file, 'w')
	    supported_found=0
            lang_setting_found=0

	    for line in loc_lines:
		if line[:10] == 'SUPPORTED=':
		    self.logger.debug (_('Replacing the value of SUPPORTED in locale conf file'))
		    setting = "SUPPORTED=" + self.parser.get_conf_val ("app_conf", "supported_languages")
		    locale_fd_wr.write( setting)
		    locale_fd_wr.write("\n")
		    supported_found = 1
                elif line[:5] == "LANG=":
                    lang_setting_found=1
		    locale_fd_wr.write(line)
		    locale_fd_wr.write("\n")
                elif line[:9] == "LANGUAGE=":
                    lang_setting_found=1
		    locale_fd_wr.write(line)
		    locale_fd_wr.write("\n")
                elif line == "\n":
		    pass
		else:
		    locale_fd_wr.write(line)
		    locale_fd_wr.write("\n")
	   
 
	    if (supported_found == 0 ):
		# SUPPORTED setting not found it locale file, add a new one.
		setting = "SUPPORTED=" + self.parser.get_conf_val ("app_conf", "supported_languages")
                locale_fd_wr.write( setting)
                locale_fd_wr.write("\n")

	    if (lang_setting_found == 0):
		# A LANG setting should be present for system-config-language to update it. So, make sure a default LANG settig is present in the i18n file.
		setting ='LANG="en_US.UTF-8"'
		locale_fd_wr.write( setting)
		locale_fd_wr.write("\n")

	    locale_fd_wr.close()

	return


    def on_Network_clicked (self, widget):
	"""Handler for Network Settings"""

	self.topWindow.hide_all()
##	Using the Empty Window as a workaround. Seems like Gtk doesn't allow all the windows to be hidden.
	self.EmptyWindow.show_all()   
	os.system("/usr/bin/nm-connection-editor > /dev/null 2>&1")
##	TODO: Error out, if nm-connection-editor is not present
	os.system("service network restart > /dev/null 2>&1")
	self.EmptyWindow.hide_all()
	self.set_title_and_desc()
	self.topWindow.show_all()
	return



    def on_Keyboard_clicked(self, widget):
	""" Handler for keyboard Settings """

        if not "/usr/share/system-config-keyboard" in sys.path:
	    sys.path.append("/usr/share/system-config-keyboard")

	self.topWindow.hide()
	import keyboard_gui
	app = keyboard_gui.childWindow()
	app.stand_alone()
	self.topWindow.show_all()
	return



#    def on_UsersButton_clicked(self, widget):
#	"""Handler for the User & Password Button"""
#
#        if not "/usr/share/system-config-users" in sys.path:
#	    sys.path.append("/usr/share/system-config-users")
#
#	self.topWindow.hide()
#	import mainWindow
#	reload(mainWindow)
#	user_win=mainWindow.mainWindow()
#	user_win.toplevel.hide()	
#	self.topWindow.show_all()
#	return



    def on_Close_clicked(self, widget):
	"""Handler for the Close button"""

	self.topWindow.hide_all()
	gtk.main_quit()
	return

    def authenticate_user(self,user_arg=None, parent_arg=None):
	""" Use the Authentication Module to authenticate the Root user here."""
	import Authentication
	
	auth_win = Authentication.Auth_Win(user=user_arg)

	if not parent_arg == None:
	    auth_win.auth_window.set_transient_for(parent_arg)
	
	auth_win.auth_window.show_all()
	gtk.main()
	return (auth_win.auth_passed(),auth_win.get_passwd())


#    def resetScript_cb(self, pid, status, data):
#	self.reset_script_complete = 1
#        self.reset_db.dialog_win.response(gtk.RESPONSE_OK)
#	return 



    def on_Reset_Button_clicked(self, widget):
        """Handler for the Reset Settings button"""

	if self.parser.get_conf_val ("reset","reboot_after_reset") == 'yes':
	    msg = _("All the settings in the appliance will be Reset to Factory Defaults and the appliance will be rebooted.\n\n Do you still wish to continue?")
	else:
	    msg = _("All the settings in the appliance will be Reset to Factory Defaults.\n\n Do you still wish to continue?")

 

	db = DialogBox.DialogBox(self.topWindow,msg)
	rc = db.display()

        if rc == gtk.RESPONSE_YES:
            # Reset the appliance here.
	    
	    (status, passwd) = self.authenticate_user(user_arg="admin",parent_arg=self.topWindow)
	    if status == False:
		## Authentication failed, don't proceed with Reset.
		self.topWindow.show_all()

	    elif status == True:
		## Continue with the Reset script. 
		self.appliance_reset_initiated=1
		self.logger.info(_("A Reset is triggered on the Appliance"))
		cmd = self.parser.get_conf_val ("reset","reset_script")

		if self.parser.get_conf_val ("reset","passwd_to_reset_script") == 'yes':
		    cmd = cmd + " " + passwd

		# Direct all the reset script output to STDOUT.
		self.reset_prc = subprocess.Popen(cmd, shell=True)
		# Waiting for the output will not show the DialogBox.
		#self.logger.info("Output from the Reset script ::\n %s",cmd_out.communicate()[0] )

		## After a Reset, also remove the eula_state file and make the user re-accept the EULA
		eula_state_file = self.parser.get_conf_val ("app_conf","eula_state")
		os.unlink (eula_state_file)


		# Not using the Reset Dialog. The user will move to the cmdline to see the output of the reset script.
		#msg = _("\nPlease wait while the appliance is being reset...")
		#self.reset_db = DialogBox.DialogBox(self.topWindow,msg,buttons=gtk.BUTTONS_OK)
                #self.topWindow.hide_all()
                #gobject.child_watch_add(cmd_out.pid,self.resetScript_cb,data=None)
                #self.reset_db.dialog_win.set_focus_on_map(True)
                #self.reset_db.display()

		if self.parser.get_conf_val ("reset","reboot_after_reset") == 'yes':
		    self.reboot_appliance = True 

		#self.reset_db.dialog_win.hide_all()
		#self.EmptyWindow.show_all() ## A fix, as Gtk need at least one window to display all the time.
		#if self.reset_script_complete == 0: # If the process hasn't terminated yet
		#    cmd_out.wait() # Wait for the Reset Script to complete

                gtk.main_quit()

        else:
	    # Reset not selected
            self.topWindow.show_all()

        return



    def on_TopWindow_destroy(self, widget):
	""" destory Top window"""

	self.topWindow.hide()
	gtk.main_quit()
	return
	
    def on_TopWindow_delete(self, widget, event, data=None):
	""" Close the window with Alt-F4"""

	self.topWindow.hide_all()
	gtk.main_quit()
	#returning False will cause the window to be closed.
	return False
	

    def destroy_cb(self, widget):
	""" Callback function to hide the main window and then terminate. """

	self.topWindow.hide_all()
	gtk.main_quit()
	return


    def grab_focus(self):
	""" Let the date Button grab the focus"""
	self.date_button.grab_focus()
	return

    def set_title_and_desc(self):
	""" Based on the settings in the conf file, set the text in the tile and description fields of the GUI"""

	## Title bar text configuration
	title_bar = self.__topwindow_tree_xml.get_widget("TopWindow")
	title_bar_text = self.parser.get_conf_val ("title", "window_title")
	if title_bar_text == "":
		title_bar_text = "Window title here..."
	title_bar.set_title(title_bar_text)


	title_label = self.__topwindow_tree_xml.get_widget("Title_Label")
	pre_markup ='<span foreground="black" weight="heavy" size ="large">'
	post_markup = '</span>'
	title_text = self.parser.get_conf_val ("title", "title_text")
	if title_text == "":
	    title_text = "Appliance Title goes here.."

	title_label.set_markup (pre_markup +title_text+ post_markup) 

	
	desc_label = self.__topwindow_tree_xml.get_widget("Description_Label")
	description_text = self.parser.get_conf_val ("description", "description_text")
	if description_text == "":
	    description_text = "Appliance description goes here.."


	
 
	pre_url_text = self.parser.get_conf_val ("url", "pre_url_text")
	url_pre_section = self.parser.get_conf_val("url","url_pre_section")
	url_post_section = self.parser.get_conf_val("url","url_post_section")

	resolv_hostname = Hostname_Win.Hostname_Win.get_resolvable_hostname()
	if resolv_hostname == "":
		resolv_hostname = "0.0.0.0"
		
	complete_url_text = pre_url_text + url_pre_section + resolv_hostname +url_post_section
	
	if  pre_url_text != "" and url_pre_section != "":
	    desc_label.set_markup (description_text +"\n"+complete_url_text)
	else:
	    desc_label.set_markup(description_text)

	return

    def select_buttons_to_display(self):
	"""Show only the selected buttons"""

	## Following tuples capture the position information while attaching buttons in ConfigTable
	bp = [ (0,1,0,1), (1,2,0,1) ,(0,1,1,2), (1,2,1,2), (0,1,2,3), (1,2,2,3), (0,1,3,4) , (1,2,3,4)]

	table = self.__topwindow_tree_xml.get_widget("ConfTable")
#	table.set_row_spacing(1,12)
#	table.set_row_spacing(2,12)
#	table.set_row_spacing(3,12)
#	table.set_col_spacing(0,7)
#	table.set_col_spacing(1,7)
	table.set_border_width(0)
	db = self.__topwindow_tree_xml.get_widget("DateButton")
	kb = self.__topwindow_tree_xml.get_widget("KeyboardButton")
	nb = self.__topwindow_tree_xml.get_widget("NetworkButton")
	hb = self.__topwindow_tree_xml.get_widget("HostnameButton")
	lb = self.__topwindow_tree_xml.get_widget("LanguageButton")
	pb = self.__topwindow_tree_xml.get_widget("PasswdButton")
	eb = self.__topwindow_tree_xml.get_widget("EULATextButton")
	rb = self.__topwindow_tree_xml.get_widget("RebootButton")
	table.remove(db)
	table.remove(kb)
	table.remove(nb)
	table.remove(hb)
	table.remove(lb)
	table.remove(pb)
	table.remove(eb)
	table.remove(rb)

	index=0

	if self.parser.get_conf_val ("visibility", "DateButton") == 'yes' :
		left_attach, right_attach, top_attach, bottom_attach = bp[index]
		table.attach(db,left_attach,right_attach,top_attach,bottom_attach,xoptions=gtk.FILL,yoptions=gtk.SHRINK, xpadding=0,ypadding=0)
		index += 1

	if self.parser.get_conf_val ("visibility", "KeyboardButton") == 'yes' :
		left_attach, right_attach, top_attach, bottom_attach = bp[index]
		table.attach(kb,left_attach,right_attach,top_attach,bottom_attach,xoptions=gtk.FILL,yoptions=gtk.SHRINK, xpadding=0,ypadding=0)
		index += 1

	if self.parser.get_conf_val ("visibility", "NetworkButton") == 'yes' :
		left_attach, right_attach, top_attach, bottom_attach = bp[index]
		table.attach(nb,left_attach,right_attach,top_attach,bottom_attach,xoptions=gtk.FILL,yoptions=gtk.SHRINK, xpadding=0,ypadding=0 )
		index += 1

	if self.parser.get_conf_val ("visibility", "HostNameButton") == 'yes' :
		left_attach, right_attach, top_attach, bottom_attach = bp[index]
		table.attach(hb,left_attach,right_attach,top_attach,bottom_attach,xoptions=gtk.FILL,yoptions=gtk.SHRINK, xpadding=0,ypadding=0 )
		index += 1

	if self.parser.get_conf_val ("visibility", "LanguageButton") == 'yes' :
		left_attach, right_attach, top_attach, bottom_attach = bp[index]
		table.attach(lb,left_attach,right_attach,top_attach,bottom_attach,xoptions=gtk.FILL, yoptions=gtk.SHRINK, xpadding=0,ypadding=0 )
		index += 1

	if self.parser.get_conf_val ("visibility", "PasswdButton") == 'yes' :
		left_attach, right_attach, top_attach, bottom_attach = bp[index]
		table.attach(pb,left_attach,right_attach,top_attach,bottom_attach,xoptions=gtk.FILL, yoptions=gtk.SHRINK,  xpadding=0,ypadding=0 )
		index += 1

	if self.parser.get_conf_val ("visibility", "EULATextButton") == 'yes' :
		left_attach, right_attach, top_attach, bottom_attach = bp[index]
		table.attach(eb,left_attach,right_attach,top_attach,bottom_attach, xoptions=gtk.FILL, yoptions=gtk.SHRINK, xpadding=0,ypadding=0)
		index += 1

	if self.parser.get_conf_val ("visibility",  "RebootButton") == 'yes' :
		left_attach, right_attach, top_attach, bottom_attach = bp[index]
		table.attach(rb,left_attach,right_attach,top_attach,bottom_attach, xoptions=gtk.FILL,yoptions=gtk.SHRINK,  xpadding=0,ypadding=0)
		index += 1
	return



    def __init__(self, parser):
	""" Import the glade file and connect the signal handlers"""
    
	self.logger = logging.getLogger("startup_app.mainWindow")

	self.parser = parser
	startup_glade_path = "/usr/share/dell_app_init/Startup_Interface.glade"
	self.logger.debug (_("Glade file of mainWindow = %s"), startup_glade_path)

	#Load the glade file
	self.__topwindow_tree_xml= gtk.glade.XML(startup_glade_path,"TopWindow")

	#Connect the signal handlers
	self.__topwindow_tree_xml.signal_autoconnect(self)
	
	## Make local references for some widgets
	self.topWindow = self.__topwindow_tree_xml.get_widget("TopWindow")
	self.date_button = self.__topwindow_tree_xml.get_widget("DateButton")


	# No need to handle signals for the Empty Window. This is fine.
	# This is just a workaround. Gtk doesn't allow hiding all the windows, when another process is running. So using an empty Window.
	emptywindow_xml_tree = gtk.glade.XML(startup_glade_path, 'EmptyWindow' )
	self.EmptyWindow =  emptywindow_xml_tree.get_widget('EmptyWindow')
	self.EmptyWindow.hide_all()


	# Import EULA Text Window, and connect the handlers
	euladisplay_xml_tree = gtk.glade.XML(startup_glade_path, 'EulaDisplayWindow')
	euladisplay_xml_tree.signal_autoconnect(self)
	self.EulaDisplayWindow = euladisplay_xml_tree.get_widget('EulaDisplayWindow')
	self.EulaTextView = euladisplay_xml_tree.get_widget('EulaDisplayText')
	self.EulaDisplayWindow.hide_all()

	if self.parser.get_conf_val ("visibility", "ResetButton") == 'no' :
		# Delete the Reset button if selected to No
		ResetBBox= self.__topwindow_tree_xml.get_widget("ResetBBox")
		ResetButton = self.__topwindow_tree_xml.get_widget("Reset_Setting")
		ResetBBox.remove(ResetButton)

	self.set_title_and_desc()
	self.select_buttons_to_display()
	
	return
