#!/usr/bin/env python

#
#  Copyright (c) 2013 Dell, Inc.
#  by Praveen K Paladugu <praveen_paladugu@dell.com>
#  Licensed under the GNU General Public license, version 2 or higher.
#


""" Dell Appliances' initial configuration program
    """
import os, sys
import subprocess
import signal
import logging
import App_Conf_Parser
import time
import gettext
import distutils.version

gettext.textdomain('dell_app_init')
gettext.bindtextdomain('dell_app_init', '/usr/share/locale')


_ = lambda x: gettext.ldgettext("dell_app_init", x)
N_ = lambda x: gettext.ldgettext("dell_app_init", x)

try:
    import pygtk
    pygtk.require('2.0')
except:
    print "Required version of PyGTK is not installed."
    sys.exit(1)



#Global variables to hold the Popen structs of X and window manager (metacity)
x_proc_id = 0
wm_proc_id = 0

#Global logger
startup_logger = 0 

#PID file
pid_file = '/var/run/dell_app_init.pid'
init_lock = 0
lock_fd = 0

# Global variable to hold the options from the conf file.
app_conf_parser = 0

# Global variable to hold the cmdline args
cmd_options = 0

def parse_cmd_options():
	"""Function to parse the command line options"""
	global cmd_options
	import optparse
	parser = optparse.OptionParser()
	parser.add_option ("-d", "--debug", help="Enable Debug logging",action="store_true",default="False", dest="debug")
	(cmd_options, args) = parser.parse_args()



def logger_start():
	"""Function to initialize the logger"""

	global startup_logger
	global cmd_options

	if ( app_conf_parser.get_conf_val ('app_conf', 'app_log') != '' ):
	    app_log_filename = app_conf_parser.get_conf_val ('app_conf', 'app_log')
	else:
	    app_log_filename = app_conf_parser.get_conf_val ('DEFAULT', 'app_log')

	if (cmd_options.debug == True):
	    logging.basicConfig(filename = app_log_filename, level = logging.DEBUG)
	else:
	    logging.basicConfig(filename = app_log_filename, level = logging.INFO)
   
 
	startup_logger = logging.getLogger("startup_app")
	startup_logger.info (_("\n\n@@@@@@@@@@Logging started at %s @@@@@@@@@@"), time.strftime("%D:%H:%M", time.localtime()))
	return
	

def start_x_wm():
	"""Function to star X Server and Window Manager (metacity)"""

	global wm_proc_id, x_proc_id, startup_logger	

	devnull = os.open('/dev/null', os.O_RDWR)

	# Configure the output file for the Xorg process.
	xorg_log_filename = app_conf_parser.get_conf_val('logging', 'xorg_log')
	if ( xorg_log_filename != "" ):
	    xorg_log =  os.open( xorg_log_filename, os.O_WRONLY|os.O_APPEND|os.O_CREAT,0644)
	    startup_logger.info (_("Using %s as the Xorg's output file."), xorg_log_filename )	
	else:
	    xorg_log_filename = app_conf_parser.get_conf_val('DEFAULT', 'xorg_log')
	    # use the Default setting
	    xorg_log =  os.open( xorg_log_filename , os.O_WRONLY|os.O_APPEND|os.O_CREAT,0644)
	    startup_logger.info (_("Using %s as the Xorg's output file."), xorg_log_filename)
    

	# Configure the output file for metacity process.
	if ( app_conf_parser.get_conf_val('logging', 'wm_log') != "" ):
	    wm_log_filename = app_conf_parser.get_conf_val('logging', 'wm_log')
	    wm_log =  os.open( wm_log_filename , os.O_WRONLY|os.O_APPEND|os.O_CREAT,0644)
	    startup_logger.info (_("Using %s as the metacity's output file."),wm_log_filename )
	
	else:
	    # use the Default setting
	    wm_log_filename = app_conf_parser.get_conf_val('DEFAULT', 'wm_log')
	    wm_log =  os.open( wm_log_filename, os.O_WRONLY|os.O_APPEND|os.O_CREAT,0644)
	    startup_logger.info (_("Using %s as the metacity's output file."), wm_log_filename)	
    

	## use the display setting from the CONF file if available.
	if (app_conf_parser.get_conf_val('app_conf','DISPLAY')):
	    os.environ['DISPLAY'] = app_conf_parser.get_conf_val('app_conf', 'DISPLAY')
	else: 
	    os.environ['DISPLAY'] = app_conf_parser.get_conf_val('DEFAULT','DISPLAY')

	startup_logger.debug (_("Using %s as the display"), os.environ['DISPLAY'])	

	## Use the VT setting from the CONF file if available.	
	if (app_conf_parser.get_conf_val('app_conf','VT')):
	    VT = app_conf_parser.get_conf_val('app_conf', 'VT')
	else: 
	    VT = app_conf_parser.get_conf_val('DEFAULT','VT')

	startup_logger.debug ("Using %s as the Virtual Terminal", VT)	

	##Start the X Server
	if not os.access( "/usr/bin/Xorg", os.X_OK):
		err =_('Xorg server not installed')
		startup_logger.error(err)
		raise RuntimeError(err)

        cmd = ['Xorg', os.environ['DISPLAY'], '-config', '/etc/dell_app_init/xorg.conf', '-ac', '-nolisten', 'tcp', VT]
	startup_logger.debug(_("Command to start Xorg process: %s"), cmd)

	try:
		x_proc_id = subprocess.Popen(cmd, stdout=xorg_log, stderr=xorg_log)
	except	OSError as e:
		err = _('Xorg server failed to start: %s' %e)
		startup_logger.error(err)
		raise RuntimeError(err)

	#Sometimes, the window Manager is starting too soon. Adding some time delay so that X server can be fully initialized for WM to detect it properly
	import time
	time.sleep(0.5)

	## Start the window Manager
	if not os.access("/usr/bin/metacity",os.X_OK):
		err =_('metacity window manager is not installed')
		startup_logger.error(err)
		raise RuntimeError(err)

	wm_cmd = ['/usr/bin/metacity', '--sm-disable', '--replace', '-d', os.getenv("DISPLAY")]
	startup_logger.debug(_("Command for metacity: %s"), wm_cmd)	
	try:
		wm_proc_id = subprocess.Popen(wm_cmd,stdout=wm_log, stderr=wm_log)
	except OSError as e:
		err = _('Cannot start Window Manager:%s' %e)
		startup_loggert.error(err)
		raise RuntimeError(err) 
	return



def kill_x_wm():
	"""Function to kill X and window Manager processes"""

	global wm_proc_id, x_proc_id, startup_logger, app_conf_parser
 
	if ( app_conf_parser.get_conf_val ("app_conf", "start_x_wm")  != 'yes'):
	    return

	if wm_proc_id:
		startup_logger.info (_("Killing the Window Manager(metacity) process"))
		startup_logger.debug (_("Process ID of the Window Manager=%d "), wm_proc_id.pid)
		wm_proc_id.terminate()
	
	if x_proc_id:	
		startup_logger.info (_("Killing the Xorg process"))
		startup_logger.debug (_("process ID of Xorg process = %d "),x_proc_id.pid)
		x_proc_id.terminate()

	# Starting the X server causing the VT to be changed to vt7 as selected above. 
	# Command to change the Virtual Terminal back to vt1.
	#TODO: Catch the exceptions as applicable.
	ret_value = os.system("/usr/bin/chvt 1")
	startup_logger.info (_("Changing the Virtual Terminal back to vt1"))
	startup_logger.debug (_("chvt command exited with: %d"),ret_value)
	return

def init_exit(return_code = 0):
    """Function to cleanup and exit"""

    global init_lock	    
    startup_logger.info(_("init_exit: Cleanup and exit"))
    kill_x_wm()

    if init_lock == 1:
	release_lock(pid_file)
    #TODO: Make sure nothing else has to be done while closing this. Close all the handlers?
    sys.exit(return_code)
    return	

def clean_and_shutdown():
    """ If the Eula is not accepted, clean the process state and shutdown the appliance. """

    global init_lock
    startup_logger.info(_("init_exit:   Cleanup and shutdown"))
    kill_x_wm()

    if init_lock == 1:
        release_lock(pid_file)
    #TODO: Make sure nothing else has to be done while closing this. Close all the handlers?
    # shutdown the appliance here. 
    os.system("shutdown -h now")
    sys.exit(1)
    return


def clean_and_reboot():
    """  """

    global init_lock
    startup_logger.info(_("init_exit:   Cleanup and Reboot"))
    kill_x_wm()

    if init_lock == 1:
        release_lock(pid_file)
    #TODO: Make sure nothing else has to be done while closing this. Close all the handlers?
    # reboot the appliance here. 
    os.system("shutdown -r now")
    sys.exit(1)
    return


def get_lock (filename):
    """ Get a file based exclusive lock """

    global lock_fd

    dir_name = os.path.dirname(filename)
    if not  os.path.exists(dir_name):
        os.makedirs(dir_name, 0755)
    try:
        lock_fd = os.open(filename, os.O_WRONLY | os.O_CREAT | os.O_EXCL, 600 )
	os.write(lock_fd, str(os.getpid()))
    except OSError, e:
        print _('Another instace running. Please try again.')
        return 0
    else:
        print 'Obtained the lock. Running..'
        return 1

     
		
    
def release_lock (filename):
    """ Release the lock """
    global lock_fd 

    #close the descriptor and del the file
    os.close(lock_fd)
    os.unlink(filename)
    return

def should_eula_be_reaccepted():
    """ Function to compare eula_state,  eula_input_version_file files to check if user has to re-accept EULA"""
    eula_state = app_conf_parser.get_conf_val ("app_conf", "eula_state")
    if (eula_state == ""):
	    ##Use the default setting
	eula_state = app_conf_parser.get_conf_val("DEFAULT", "eula_state")    

    eula_input = app_conf_parser.get_conf_val ("app_conf","eula_input_version_file")

    if (eula_input == ""):
	#eula_input file not defined, force the user to accept EULA
	return 1

    installed_version = app_conf_parser.get_eula_version_from_file (eula_input)
    if (installed_version == -1 or installed_version == 0 ):
	#eula_input file not found, have to accept EULA
	return 1
    
    saved_version = app_conf_parser.get_eula_version_from_file (eula_state)
    if (saved_version == 0 or saved_version == -1):
	# No previous saved version found or state file not found, re-accept EULA
	return 1

    if distutils.version.StrictVersion (installed_version) > distutils.version.StrictVersion (saved_version):
	# Doesn't make sense
	return 1
    elif distutils.version.StrictVersion (installed_version) ==  distutils.version.StrictVersion (saved_version):
	#Only if the saved version matches the installed version, no need to re-accept EULA
	return 0
    else:
	return 1


def sig_handler(signum, frame):
        #Clean the pid before exiting when user sends a signal.
        print "Received Signal:", signum
        init_exit(1)

    
def import_req_modules():

	global startup_logger
	global EulaBox, Startup_Interface, gtk, gtk_glade
     
	#NOTE: Gtk and Glade libraries can be imported only after a DISPLAY is setup. So import here only.
	try:
    		import gtk as gtk
    		import gtk.glade as gtk_glade
	except:
    		err = _("Gtk/Glade is not installed. Please check if X server is running and DISPLAY is set.") 
    		print err
		startup_logger.error (err)
    		init_exit(1)


	# After gtk is imported, set the domain for gettext.
	gtk_glade.bindtextdomain('dell_app_init', '/usr/share/locale')
	gtk_glade.bindtextdomain('test')


	#NOTE: The below classes also import gtk and glade libraries. Have to be imported here only.
	try:
    		import EulaBox as EulaBox
	except:
    		err = _("Cannot import EulaBox module.")
		print err
		startup_logger.error(err)   
    		init_exit(1)    


	try:
    		import Startup_Interface as Startup_Interface
	except:
    		err = _("Cannot import Startup_Interface module.")
		print err
		startup_logger.error(err)
    		init_exit(1)    

	return


def my_main():
	global app_conf_parser, init_lock

	## If not root user, exit.
	if (os.geteuid() != 0 ):
	    print _('Not root!! This program has to be triggered by the root user only.')
	    sys.exit(1)


	## Get and check lock
	init_lock = get_lock (pid_file)
	if init_lock ==  0:
	    sys.exit(1)	
	

	#TODO: Enable Internalization
	import locale
	locale.setlocale(locale.LC_ALL, "")

	# Initialize the parser.
	app_conf_parser = App_Conf_Parser.App_Conf_Parser("/etc/dell_app_init/dell_app_init.conf /etc/dell_app_init/conf.d/*.conf")

	#parse command line options
	parse_cmd_options()
    
	#start the logger
	logger_start()


	# start an X server and window manager only if selected yes. If not, the DISPLAY envi variable has to be set. 
	if ( app_conf_parser.get_conf_val ("app_conf", "start_x_wm")  == 'yes'):
	    start_x_wm()

	# Import all the required modules, after X & WM are started successfully. 
	import_req_modules()
	
	if (should_eula_be_reaccepted() == 1 ):
	    ## Input has to be the file containing the EULA text
	    eula_text_filename = app_conf_parser.get_eula_text_filename()
	    if ( eula_text_filename ) and os.access (eula_text_filename, os.R_OK):
		eula_box = EulaBox.EulaBox(eula_text_filename, app_conf_parser)
		eula_box.grab_focus()
		#eula_box.eula_dialog.hide_all()
		eula_box.eula_window.show_all()
		gtk.main()
    
	    else:
		print (_('EULA text file is not available. Cannot proceed.'))
		startup_logger.error (_('EULA text file is not available. Cannot proceed.'))	    
		init_exit(1)


	    if (not eula_box.is_eula_accepted()):
		startup_logger.error(_('EULA Not accepted. So closing the application and shutting down'))
		clean_and_shutdown()
 	


	#After accepting the EULA, continue to the main conf Window.
    	startup = Startup_Interface.Startup_Interface(app_conf_parser)
	startup.grab_focus()
    	gtk.main()

	if startup.appliance_reset_initiated == 1:  # Appliance reset
 
	    kill_x_wm()
	    print "Waiting for the reset script to complete..."
	    startup.reset_prc.wait()
 
	    if startup.reboot_appliance == True:
		if init_lock == 1:
		    release_lock(pid_file)
		startup_logger.info (_("User initiated a reboot. Rebooting now."))
		os.system("shutdown -r now")
		sys.exit(1)
	    else:
		if init_lock == 1:
		    release_lock(pid_file)
		startup_logger.info(_("Exiting the main function after reset"))
		sys.exit(0)

	else: 
	    if startup.reboot_appliance == True:
		startup_logger.info (_("User initiated a reboot. Rebooting now."))
		clean_and_reboot()


	startup_logger.info(_("Exiting the main function"))
    	init_exit(0)



if __name__ == '__main__':
#Main function

    signal.signal (signal.SIGTERM, sig_handler)
    try:
	my_main()
    except KeyboardInterrupt, e:
        print( _("Key board Exception received") )
	init_exit(1)
